"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = NetlifyCmsModule;
Object.defineProperty(exports, "meta", {
  enumerable: true,
  get: function get() {
    return _package.default;
  }
});

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _path = require("path");

var _fsExtra = require("fs-extra");

var _lodash = _interopRequireDefault(require("lodash"));

var _common = require("@nuxt/common");

var _consola = _interopRequireDefault(require("consola"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _stdEnv = _interopRequireDefault(require("std-env"));

var _pify = _interopRequireDefault(require("pify"));

var _webpack = _interopRequireDefault(require("webpack"));

var _webpackDevMiddleware = _interopRequireDefault(require("webpack-dev-middleware"));

var _webpackHotMiddleware = _interopRequireDefault(require("webpack-hot-middleware"));

var _webpackbar = _interopRequireDefault(require("webpackbar"));

var _serveStatic = _interopRequireDefault(require("serve-static"));

var _package = _interopRequireDefault(require("../package.json"));

var _configManager = _interopRequireDefault(require("./configManager"));

var _webpack2 = _interopRequireDefault(require("./webpack.config"));

var _yaml = require("./utils/yaml");

/* eslint-disable import/no-extraneous-dependencies */

/* covered by nuxt */
var logger = _consola.default.withScope("nuxt:netlify-cms");

var WEBPACK_CLIENT_COMPILER_NAME = "client";
var WEBPACK_NETLIFY_COMPILER_NAME = "netlify-cms";
var NETLIFY_CONFIG_FILE_NAME = "config.yml";

function NetlifyCmsModule(moduleOptions) {
  var _this = this;

  var configManager = new _configManager.default(this.options, moduleOptions);
  var config = configManager.config;

  var emitNetlifyConfig = function emitNetlifyConfig(compilation) {
    var netlifyConfigYAML = (0, _yaml.toYAML)(configManager.cmsConfig);
    compilation.assets[NETLIFY_CONFIG_FILE_NAME] = {
      source: function source() {
        return netlifyConfigYAML;
      },
      size: function size() {
        return netlifyConfigYAML.length;
      }
    };
  }; // This will be called once when builder started


  this.nuxt.hook("build:before", function (builder) {
    var bundleBuilder = builder.bundleBuilder;
    var webpackConfig = (0, _webpack2.default)(WEBPACK_NETLIFY_COMPILER_NAME, _this.options, config);
    webpackConfig.plugins.push({
      apply: function apply(compiler) {
        compiler.hooks.emit.tapAsync("NetlifyCMSPlugin", function (compilation, cb) {
          compilation.hooks.additionalAssets.tapAsync("NetlifyCMSPlugin", function (callback) {
            emitNetlifyConfig(compilation);
            callback();
          });
          emitNetlifyConfig(compilation);
          cb();
        });
      }
    });
    !_this.options.dev && webpackConfig.plugins.push(new _webpackbar.default({
      name: WEBPACK_NETLIFY_COMPILER_NAME,
      color: "red",
      reporters: ["basic", "fancy", "profile", "stats"],
      basic: !_this.options.build.quiet && _stdEnv.default.minimalCLI,
      fancy: !_this.options.build.quiet && !_stdEnv.default.minimalCLI,
      profile: !_this.options.build.quiet && _this.options.build.profile,
      stats: !_this.options.build.quiet && !_this.options.dev && _this.options.build.stats,
      reporter: {
        change: function change(_, _ref) {
          var shortPath = _ref.shortPath;

          _this.nuxt.callHook("bundler:change", shortPath);
        },
        done: function done(context) {
          if (context.hasErrors) {
            _this.nuxt.callHook("bundler:error");
          }
        },
        allDone: function allDone() {
          _this.nuxt.callHook("bundler:done");
        }
      }
    }));
    var netlifyCompiler = (0, _webpack.default)(webpackConfig); // This will be run just before webpack compiler starts

    _this.nuxt.hook("build:compile", function (_ref2) {
      var name = _ref2.name;

      if (name !== WEBPACK_CLIENT_COMPILER_NAME) {
        return;
      }

      logger.success("Netlify-cms builder initialized"); // This will be run just after webpack compiler ends

      netlifyCompiler.hooks.done.tapAsync("NetlifyCMSPlugin",
      /*#__PURE__*/
      function () {
        var _ref3 = (0, _asyncToGenerator2.default)(
        /*#__PURE__*/
        _regenerator.default.mark(function _callee(stats, cb) {
          return _regenerator.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!stats.hasErrors()) {
                    _context.next = 2;
                    break;
                  }

                  return _context.abrupt("return");

                case 2:
                  // Show a message inside console when the build is ready
                  _this.options.dev && logger.info("Netlify-cms served on: ".concat(config.adminPath));
                  cb();

                case 4:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));

        return function (_x, _x2) {
          return _ref3.apply(this, arguments);
        };
      }()); // in development

      if (_this.options.dev) {
        // Use shared filesystem and cache
        netlifyCompiler.outputFileSystem = bundleBuilder.mfs; // Create webpack dev middleware

        var netlifyWebpackDevMiddleware = (0, _pify.default)((0, _webpackDevMiddleware.default)(netlifyCompiler, {
          publicPath: "/",
          stats: false,
          logLevel: "silent",
          watchOptions: _this.options.watchers.webpack
        }));
        netlifyWebpackDevMiddleware.close = (0, _pify.default)(netlifyWebpackDevMiddleware.close); // Create webpack hot middleware

        var netlifyWebpackHotMiddleware = (0, _pify.default)((0, _webpackHotMiddleware.default)(netlifyCompiler, {
          log: false,
          heartbeat: 1000
        })); // Inject to renderer instance

        if (builder.nuxt.renderer) {
          builder.nuxt.renderer.netlifyWebpackDevMiddleware = netlifyWebpackDevMiddleware;
          builder.nuxt.renderer.netlifyWebpackHotMiddleware = netlifyWebpackHotMiddleware;
        } // Stop webpack middleware on nuxt.close()


        _this.nuxt.hook("close",
        /*#__PURE__*/
        (0, _asyncToGenerator2.default)(
        /*#__PURE__*/
        _regenerator.default.mark(function _callee2() {
          return _regenerator.default.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  _context2.next = 2;
                  return _this.nuxt.renderer.netlifyWebpackDevMiddleware.close();

                case 2:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        })));
      } else {
        // Only run the compiler in production,
        // in dev build is started by dev-middleware hooked to client webpack compiler
        _this.nuxt.hook("build:done",
        /*#__PURE__*/
        (0, _asyncToGenerator2.default)(
        /*#__PURE__*/
        _regenerator.default.mark(function _callee3() {
          return _regenerator.default.wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  _context3.next = 2;
                  return new Promise(function (resolve, reject) {
                    netlifyCompiler.run(function (err, stats) {
                      /* istanbul ignore next */
                      if (err) {
                        return reject(err);
                      } else if (stats.hasErrors()) {
                        if (_this.options.test) {
                          err = stats.toString(_this.options.build.stats);
                        }

                        return reject(err);
                      }

                      resolve();
                    });
                  });

                case 2:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3);
        })));
      }
    });
  }); // Serve netlify CMS

  if (this.options.dev) {
    // Insert webpackDevMiddleware to serve netlify CMS in development
    this.addServerMiddleware({
      path: config.adminPath,
      handler: function () {
        var _handler = (0, _asyncToGenerator2.default)(
        /*#__PURE__*/
        _regenerator.default.mark(function _callee4(req, res) {
          return _regenerator.default.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (!_this.nuxt.renderer.netlifyWebpackDevMiddleware) {
                    _context4.next = 4;
                    break;
                  }

                  logger.info("Netlify-cms requested url: ".concat((0, _common.urlJoin)(config.adminPath, req.url)));
                  _context4.next = 4;
                  return _this.nuxt.renderer.netlifyWebpackDevMiddleware(req, res);

                case 4:
                  if (!_this.nuxt.renderer.netlifyWebpackHotMiddleware) {
                    _context4.next = 7;
                    break;
                  }

                  _context4.next = 7;
                  return _this.nuxt.renderer.netlifyWebpackHotMiddleware(req, res);

                case 7:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4);
        }));

        function handler(_x3, _x4) {
          return _handler.apply(this, arguments);
        }

        return handler;
      }()
    }); // Start watching config file

    var patterns = [(0, _common.r)(configManager.cmsConfigFileName)];
    var options = (0, _objectSpread2.default)({}, this.options.watchers.chokidar, {
      ignoreInitial: true
    });

    var refreshFiles = _lodash.default.debounce(function () {
      configManager.readCmsConfigFile();

      _this.nuxt.renderer.netlifyWebpackDevMiddleware.invalidate();

      _this.nuxt.renderer.netlifyWebpackHotMiddleware.publish({
        action: "reload"
      });

      logger.info("Netlify-cms files refreshed");
    }, 200); // Watch for src Files


    var fileWatcher = _chokidar.default.watch(patterns, options).on("add", refreshFiles).on("change", refreshFiles).on("unlink", refreshFiles);

    this.nuxt.renderer.netlifyFileWatcher = fileWatcher; // Stop watching on nuxt.close()

    this.nuxt.hook("close", function () {
      _this.nuxt.renderer.netlifyFileWatcher.close();
    });
  } else {
    // Statically serve netlify CMS (i.e. .nuxt/dist/admin/) files in production
    this.addServerMiddleware({
      path: config.adminPath,
      handler: (0, _serveStatic.default)(config.buildDir, {
        maxAge: "1y" // 1 year in production

      })
    });
  } // Move cms folder from `.nuxt/dist/admin` folder to `dist/` after nuxt generate


  this.nuxt.hook("generate:distCopied",
  /*#__PURE__*/
  function () {
    var _ref6 = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee5(nuxt) {
      return _regenerator.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return (0, _fsExtra.copy)((0, _path.resolve)(nuxt.options.buildDir, "dist", config.adminPath).replace(/\/$/, ""), (0, _path.join)(nuxt.distPath, config.adminPath).replace(/\/$/, ""));

            case 2:
              logger.success("Netlify-cms files copied");

            case 3:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    }));

    return function (_x5) {
      return _ref6.apply(this, arguments);
    };
  }());
} // REQUIRED if publishing as an NPM package