"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = webpackNetlifyCmsConfig;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _fs = require("fs");

var _path = require("path");

var _extractCssChunksWebpackPlugin = _interopRequireDefault(require("extract-css-chunks-webpack-plugin"));

var _friendlyErrorsWebpackPlugin = _interopRequireDefault(require("@nuxt/friendly-errors-webpack-plugin"));

var _common = require("@nuxt/common");

var _webpack = _interopRequireDefault(require("webpack"));

var _htmlWebpackPlugin = _interopRequireDefault(require("html-webpack-plugin"));

/* eslint-disable import/no-extraneous-dependencies */

/* covered by nuxt */
function webpackNetlifyCmsConfig(name, nuxtOptions, moduleConfig) {
  var BUILD_MODE = nuxtOptions ? "development" : "production";
  var ENTRY = (0, _path.resolve)(__dirname, "../lib/entry");
  var BUILD_DIR = moduleConfig.buildDir;
  var CHUNK_FILENAME = nuxtOptions.build.filenames.chunk({
    isDev: nuxtOptions.dev,
    isModern: nuxtOptions.modern
  });
  var PUBLIC_PATH = (0, _common.urlJoin)(nuxtOptions.router.base, moduleConfig.adminPath);
  var EXTENSIONS_DIR = moduleConfig.moduleConfigDir;
  var PAGE_TITLE = moduleConfig.adminTitle;
  var PAGE_TEMPLATE = (0, _path.resolve)(__dirname, "../lib/template", "index.html");
  var REQUIRE_EXTENSIONS = (0, _fs.existsSync)(EXTENSIONS_DIR) ? true : false;
  var HMR_CLIENT = (0, _path.resolve)(__dirname, "../lib/hmr.client");
  var CSS_FILE = "netlify-cms/dist/cms.css";
  var REQUIRE_CSS = (0, _fs.existsSync)((0, _path.resolve)(__dirname, "node_modules", CSS_FILE));
  var CSS_FILENAME = "style.[contenthash].css";
  var config = {
    name: name,
    mode: BUILD_MODE,
    entry: {
      app: ENTRY
    },
    output: {
      path: BUILD_DIR,
      filename: "bundle.[hash].js",
      chunkFilename: CHUNK_FILENAME,
      publicPath: PUBLIC_PATH
    },
    module: {
      rules: [{
        test: /\.css$/,
        loader: "style-loader!css-loader"
      }, {
        test: /\.(eot|svg|ttf|woff|woff2)$/,
        loader: "url-loader",
        options: {
          limit: 1000,
          // 1 KO
          name: "public/fonts/[name].[hash:7].[ext]"
        }
      }]
    },
    resolve: {
      alias: {
        extensions: EXTENSIONS_DIR
      }
    },
    plugins: [].concat((0, _toConsumableArray2.default)(nuxtOptions.build.extractCSS ? [new _extractCssChunksWebpackPlugin.default(Object.assign({
      filename: CSS_FILENAME,
      chunkFilename: CSS_FILENAME,
      // TODO: https://github.com/faceyspacey/extract-css-chunks-webpack-plugin/issues/132
      reloadAll: true
    }, nuxtOptions.build.extractCSS))] : []), [new _htmlWebpackPlugin.default({
      title: PAGE_TITLE,
      filename: "index.html",
      template: PAGE_TEMPLATE,
      inject: true,
      chunksSortMode: "dependency"
    }), new _webpack.default.DefinePlugin({
      REQUIRE_EXTENSIONS: REQUIRE_EXTENSIONS,
      REQUIRE_CSS: REQUIRE_CSS,
      CSS_FILE: CSS_FILE
    })])
  }; // --------------------------------------
  // Development specific config
  // --------------------------------------

  if (nuxtOptions.dev) {
    // Add HMR support
    config.entry.app = [HMR_CLIENT, config.entry.app];
    config.plugins.push(new _webpack.default.HotModuleReplacementPlugin(), new _webpack.default.NoEmitOnErrorsPlugin(), // https://webpack.js.org/plugins/named-modules-plugin
    new _webpack.default.NamedModulesPlugin()); // Add friendly error plugin

    if (!nuxtOptions.build.quiet && nuxtOptions.build.friendlyErrors) {
      config.plugins.push(new _friendlyErrorsWebpackPlugin.default({
        clearConsole: false,
        reporter: "consola",
        logLevel: "WARNING"
      }));
    }
  } else {
    // --------------------------------------
    // Production specific config
    // --------------------------------------
    // Minify and optimize the JavaScript
    config.plugins.push( // This is needed in webpack 2 for minify CSS
    new _webpack.default.LoaderOptionsPlugin({
      minimize: true
    }));
  }

  return config;
}